from mlfields.data_models import (
    db,
    FeatureEvaluations
)
from flask_restful import (
    Resource,
    reqparse
)


parser = reqparse.RequestParser()
parser.add_argument('feature_id')
parser.add_argument('metric_id')
parser.add_argument('value')


class FEs(Resource):
    def post(self, project_id, fm_id):
        args = parser.parse_args()
        new_fe = FeatureEvaluations(
            fm_id=fm_id,
            feature_id=args["feature_id"],
            metric_id=args["metric_id"],
            value=args["value"]
        )
        db.session.add(new_fe)
        db.session.commit()
        return {}, 201



from datetime import datetime

import sqlalchemy
from flask_sqlalchemy import SQLAlchemy


db = SQLAlchemy()


class Projects(db.Model):
    __tablename__ = "projects"
    project_id = db.Column(db.Integer, primary_key=True)
    name = db.Column(db.Integer, unique=True, nullable=False)
    note = db.Column(db.String)
    created = db.Column(db.DateTime, nullable=False, default=sqlalchemy.sql.func.now())


class FeatureDefinitions(db.Model):
    __tablename__ = "feature_definitions"
    feature_id = db.Column(db.Integer, primary_key=True)
    project_id = db.Column(db.Integer, db.ForeignKey("projects.project_id"), nullable=False)
    name = db.Column(db.String, nullable=False)
    created = db.Column(db.DateTime, nullable=False, default=sqlalchemy.sql.func.now())
    note = db.Column(db.String)


class FeatureMatrices(db.Model):
    __tablename__ = "feature_matrices"
    fm_id = db.Column(db.Integer, primary_key=True)
    project_id = db.Column(db.Integer, db.ForeignKey("projects.project_id"), nullable=False)
    created = db.Column(db.DateTime, nullable=False, default=sqlalchemy.sql.func.now())
    name = db.Column(db.String, nullable=False)
    note = db.Column(db.String)


class FeatureListInFM(db.Model):
    __tablename__ = "feature_list_in_fm"
    row_id = db.Column(db.Integer, primary_key=True)
    fm_id = db.Column(db.Integer, db.ForeignKey("feature_matrices.fm_id"), nullable=False)
    feature_id = db.Column(db.Integer, db.ForeignKey("feature_definitions.feature_id"), nullable=False)


class FeatureEvaluations(db.Model):
    __tablename__ = "feature_evaluation"
    row_id = db.Column(db.Integer, primary_key=True)
    feature_id = db.Column(db.Integer, db.ForeignKey("feature_definitions.feature_id"), nullable=False)
    fm_id = db.Column(db.Integer, db.ForeignKey("feature_matrices.fm_id"), nullable=False)
    metric_id = db.Column(db.Integer, db.ForeignKey("evaluation_metrics.metric_id"), nullable=False)
    value = db.Column(db.Numeric)


class EvaluationMetrics(db.Model):
    __tablename__ = "evaluation_metrics"
    metric_id = db.Column(db.Integer, primary_key=True)
    project_id = db.Column(db.Integer, db.ForeignKey("projects.project_id"), nullable=False)
    metric_name = db.Column(db.String, nullable=False)
    __table_args__ = (db.UniqueConstraint("project_id", "metric_name", name="_project_id_metric_name_uc"),)



import requests


class MLFieldSession(object):
    def __init__(self, host):
        self.session = requests.Session()
        self.url_header = f"http://{host}:5000/api"
        self.project_id = None

    def set_project(self, name, note=""):
        self.project_id = self.get_or_create_project(name, note)

    def get_or_create_project(self, name, note):
        projects = self.session.get(f"{self.url_header}/projects")
        existing_projects = [p for p in projects if p.name == name]
        if existing_projects:
            return existing_projects[0].project_id
        else:
            res = self.session.post(f"{self.url_header}/projects", json=dict(name=name, note=note))
            return res.json()["project_id"]

    def check_project(self):
        def deco(func):
            if not self.project_id:
                raise Exception("Project has not been set")
            return func
        return deco

    @self.check_project
    def register_feature_matrix(self, name, note, df):
        fm_id = self.post_fm(name, note)

    @self.check_project
    def post_fm(self, name, note=""):
        res = self.session.post(
            f"{self.url_header}/projects/{self.project_id}/fms", json=dict(name=name, note=note)
        )
        return res.json()["fm_id"]

    @self.check_project
    def post_fd(self, name, fm_id, note=""):
        pass



from flask import (
    Blueprint,
    flash,
    redirect,
    render_template,
    request,
    url_for
)

from mlfields.data_models import (
    db,
    Projects
)


bp = Blueprint('projects', __name__)


@bp.route('/projects/create/', methods=('GET', 'POST'))
def create():
    if request.method == 'POST':
        name = request.form['name']
        note = request.form['note']
        error = None
        if not name:
            error = 'Name is required.'
        if error is not None:
            flash(error)
        else:
            res = requests.post(url_for("projects.list"), json={"name": name, "note": note})
            return redirect(url_for("projects.list"))
    return render_template('projects/create.html')


@bp.route('/projects/', methods=('GET',))
def list():
    projects = Projects.query
    urls = [url_for("feature_matrices.list", project_id=p.project_id) for p in projects]
    return render_template('projects/list.html', contexts=zip(projects, urls))


@bp.route('/', methods=('GET',))
def index():
    return redirect(url_for('projects.list'))



from flask import (
    Blueprint,
    render_template,
    url_for,
    request
)

from mlfields.data_models import (
    Projects,
    FeatureDefinitions
)


bp = Blueprint('feature_definitions', __name__)
@bp.route('', methods=('GET',))
def list(project_id):
    project = Projects.query.filter_by(project_id=project_id).first()
    fds = FeatureDefinitions.query.filter_by(project_id=project_id)
    return render_template('feature_definitions/list.html', project=project, fds=fds)



from mlfields.data_models import (
    db,
    FeatureDefinitions
)
from flask_restful import (
    Resource,
    reqparse
)


parser = reqparse.RequestParser()
parser.add_argument('name')
parser.add_argument('note')


class FDs(Resource):
    def post(self, project_id):
        args = parser.parse_args()
        new_fd = FeatureDefinitions(
            project_id=project_id,
            name=args["name"],
            note=args["note"]
        )
        db.session.add(new_fd)
        db.session.commit()
        return {"feature_id": new_fd.feature_id}, 201

    def get(self, project_id):
        fds = FeatureDefinitions.query.filter_by(project_id=project_id)
        return [
            dict(
                feature_id=fd.feature_id,
                name=fd.name,
                note=fd.note
            )
            for fd in fds
        ]



from flask import (
    Blueprint,
    render_template,
    url_for
)

from mlfields.data_models import (
    Projects,
    FeatureMatrices
)


bp = Blueprint('feature_matrices', __name__)

@bp.route('', methods=('GET',))
def list(project_id):
    project = Projects.query.filter_by(project_id=project_id).first()
    fms = FeatureMatrices.query.filter_by(project_id=project_id)
    urls = [url_for("feature_evaluations.list", project_id=project.project_id, fm_id=fm.fm_id) for fm in fms]
    return render_template('feature_matrices/list.html', project=project, contexts=zip(fms, urls))



from flask import current_app
from flask_restful import (
    Resource,
    reqparse
)

from mlfields.data_models import (
    db,
    Projects
)


parser = reqparse.RequestParser()
parser.add_argument('name', location=['json', 'args'])
parser.add_argument('note', location=['json', 'args'])


class PJs(Resource):
    def post(self):
        args = parser.parse_args()
        new_project = Projects(name=args['name'], note=args['note'])
        db.session.add(new_project)
        db.session.commit()
        return {"project_id": new_project.project_id}, 201

    def get(self):
        projects = Projects.query
        return [
            {
                "project_id": p.project_id,
                "name": p.name,
                "created": p.created.strftime('%Y-%m-%d %H:%M:%S'),
                "note": p.note
            }
            for p in projects
        ]



from flask import (
    Blueprint,
    render_template,
    url_for
)
import pandas

from mlfields.data_models import (
    Projects,
    FeatureMatrices,
    FeatureDefinitions,
    FeatureEvaluations,
    EvaluationMetrics
)


bp = Blueprint('feature_evaluations_for_each_feature', __name__)


@bp.route('', methods=('GET',))
def list(project_id, feature_id):
    project = Projects.query.filter_by(project_id=project_id).first()
    fes = FeatureEvaluations.query.filter_by(feature_id=feature_id)
    fms = FeatureMatrices.query
    fd = FeatureDefinitions.query.filter_by(feature_id=feature_id).first()
    metrics = EvaluationMetrics.query.filter_by(project_id=project_id)
    df = pandas.read_sql(fes.statement, fes.session.bind)
    metric_name_df = pandas.read_sql(metrics.statement, metrics.session.bind)[["metric_id", "metric_name"]]
    fms_df = pandas.read_sql(fms.statement, fms.session.bind)[["fm_id", "name", "created"]]
    eval_df = df.merge(metric_name_df, on="metric_id", how="left")
    eval_df = eval_df.pivot(index=["fm_id"], columns="metric_name", values="value").reset_index()
    eval_df["fm_id"] = eval_df["fm_id"].astype('int64')
    eval_df = eval_df.merge(fms_df, on="fm_id", how="left")
    return render_template(
        'feature_evaluations/list_for_each_feature.html', 
        project=project, fd=fd, metrics=metric_name_df["metric_name"].values, eval_df=eval_df
    )




from mlfields.data_models import (
    db,
    EvaluationMetrics
)
from flask_restful import (
    Resource,
    reqparse
)


parser = reqparse.RequestParser()
parser.add_argument('name')


class Metrics(Resource):
    def post(self, project_id):
        args = parser.parse_args()
        new_metric = EvaluationMetrics(
            project_id=project_id,
            metric_name=args["name"]
        )
        db.session.add(new_metric)
        db.session.commit()
        return {"metric_id": new_metric.metric_id}, 201

    def get(self, project_id):
        metrics = EvaluationMetrics.query.filter_by(project_id=project_id)
        return [
            dict(
                metric.metric_id,
                metric.metric_name
            )
            for metric in metrics
        ]



from mlfields.data_models import (
    db,
    FeatureMatrices
)
from flask_restful import (
    Resource,
    reqparse
)


parser = reqparse.RequestParser()
parser.add_argument('name')
parser.add_argument('note')


class FMs(Resource):
    def post(self, project_id):
        args = parser.parse_args()
        new_fm = FeatureMatrices(
            project_id=project_id,
            name=args["name"],
            note=args["note"]
        )
        db.session.add(new_fm)
        db.session.commit()
        return {"fm_id": new_fm.fm_id}, 201



import os

import click
from flask import (
    Flask,
    current_app
)
from flask.cli import with_appcontext
from flask_restful import Api


def create_app(test_config=None):
    app = Flask(__name__, instance_relative_config=True)
    db_path = "{}/mlfields.db".format(app.instance_path)
    app.config.from_mapping(
        SECRET_KEY='dev',
        SQLALCHEMY_DATABASE_URI = 'sqlite:///{}'.format(db_path)
    )

    if test_config is None:
        app.config.from_pyfile('config.py', silent=True)
    else:
        app.config.from_mapping(test_config)

    try:
        os.makedirs(app.instance_path)
    except OSError:
        pass

    @app.route('/hello')
    def hello():
        return 'Hello, World'

    from .data_models import db
    db.init_app(app)
    add_init_db_command(app)

    from . import project
    app.register_blueprint(project.bp)

    from . import feature_matrices
    app.register_blueprint(feature_matrices.bp, url_prefix='/projects/<int:project_id>/fms/')

    from . import feature_definitions
    app.register_blueprint(feature_definitions.bp, url_prefix='/projects/<int:project_id>/fds/')

    from . import feature_evaluations
    app.register_blueprint(feature_evaluations.bp, url_prefix='/projects/<int:project_id>/fms/<int:fm_id>/')

    from . import feature_evaluations_for_each_feature
    app.register_blueprint(feature_evaluations_for_each_feature.bp, url_prefix='/projects/<int:project_id>/fds/<int:feature_id>/')

    from .apis import (
        pjs,
        fds,
        fms,
        metrics,
        fes
    )
    api = Api(app)
    api.add_resource(pjs.PJs, '/api/projects/')
    api.add_resource(fds.FDs, '/api/projects/<int:project_id>/fds/')
    api.add_resource(fms.FMs, '/api/projects/<int:project_id>/fms/')
    api.add_resource(metrics.Metrics, '/api/projects/<int:project_id>/metrics/')
    api.add_resource(fes.FEs, '/api/projects/<int:project_id>/fms/<int:fm_id>/')

    return app


@click.command('init-db')
@with_appcontext
def init_db_command():
    init_db()
    click.echo('Initialized the database.')


def add_init_db_command(app):
    app.cli.add_command(init_db_command)


def init_db():
    from .data_models import db
    db = data_models.db
    with current_app.app_context():
        db.create_all()
